package geniusweb.party.inform;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;

import org.junit.Test;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.PartyId;
import geniusweb.inform.Agreements;
import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import tudelft.utilities.junit.GeneralTests;

public class AgreementsTest extends GeneralTests<Agreements> {
	private final ObjectMapper jackson = new ObjectMapper();
	private String jsonstring = "{\"party2\":{\"issuevalues\":{\"i1\":\"v1\"}},\"party1\":{\"issuevalues\":{\"i1\":\"v1\"}}}";

	private Bid bid1 = new Bid("i1", new DiscreteValue("v1"));
	private Bid bid2 = new Bid("i1", new DiscreteValue("v2"));
	private PartyId party1 = new PartyId("party1");
	private PartyId party2 = new PartyId("party2");
	private PartyId party3 = new PartyId("party3");
	private Agreements agree1 = new Agreements().with(bid1,
			new HashSet<>(Arrays.asList(party1, party2)));
	private Agreements agree1a = new Agreements().with(bid1,
			new HashSet<>(Arrays.asList(party1, party2)));
	private Agreements agree1b = new Agreements().with(bid1,
			new HashSet<>(Arrays.asList(party2, party1)));

	private Agreements agree2 = new Agreements().with(bid2,
			new HashSet<>(Arrays.asList(party1, party2)));
	private Agreements agree3 = new Agreements().with(bid1,
			new HashSet<>(Arrays.asList(party1, party2, party3)));

	@Override
	public List<List<Agreements>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(agree1, agree1a, agree1b),
				Arrays.asList(agree2), Arrays.asList(agree3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList(
				"Agreements.*party2.*Bid.*i1=\"v1\".*party1=Bid.*i1=\"v1\".*",
				"Agreements.*party2.*Bid.*i1=\"v2\".*party1=Bid.*i1=\"v2\".*",
				"Agreements.*party2.*Bid.*i1=\"v1\".*party1=Bid.*i1=\"v1\".*party3=Bid.*i1=\\\"v1\\\".*");
	}

	@Test
	public void serializeTest() throws JsonProcessingException {
		System.out.println(jackson.writeValueAsString(agree1));
		assertEquals(jsonstring, jackson.writeValueAsString(agree1));
	}

	@Test
	public void deserializeTest() throws IOException {
		Agreements act = jackson.readValue(jsonstring, Agreements.class);
		assertEquals(agree1, act);
	}
}
