package geniusweb.party.inform;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.math.BigDecimal;
import java.net.URISyntaxException;
import java.util.Arrays;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;

import org.junit.Before;
import org.junit.Test;

import com.fasterxml.jackson.core.JsonParseException;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonMappingException;
import com.fasterxml.jackson.databind.ObjectMapper;

import geniusweb.actions.PartyId;
import geniusweb.inform.Agreements;
import geniusweb.inform.Finished;
import geniusweb.inform.Inform;
import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.NumberValue;
import geniusweb.issuevalue.Value;
import tudelft.utilities.junit.GeneralTests;

public class FinishedTest extends GeneralTests<Finished> {

	private Finished finished1, finished1a, finished2, finished3;

	private String asJson = "{\"Finished\":{\"agreements\":{\"party2\":{\"issuevalues\":{\"issue3\":9012,\"issue2\":1,\"issue1\":\"b\"}},\"party1\":{\"issuevalues\":{\"issue3\":9012,\"issue2\":1,\"issue1\":\"b\"}}}}}";

	private PartyId party1 = new PartyId("party1");
	private PartyId party2 = new PartyId("party2");

	@Before
	public void before() throws URISyntaxException {
		final String ISSUE1 = "issue1";
		final String ISSUE2 = "issue2";
		final Value VALUE1 = new DiscreteValue("value1");
		final Value VALUE2 = new NumberValue("10");

		Map<String, Value> issuevalues = new HashMap<>();
		issuevalues.put(ISSUE1, new DiscreteValue("b"));
		issuevalues.put(ISSUE2, new NumberValue(BigDecimal.ONE));
		issuevalues.put("issue3", new NumberValue(new BigDecimal("9012")));
		Bid bid = new Bid(issuevalues);

		Map<String, Value> issuevaluesb = new HashMap<>();
		issuevaluesb.put(ISSUE1, new DiscreteValue("b"));
		issuevaluesb.put(ISSUE2, new NumberValue(BigDecimal.ONE));
		Bid bidb = new Bid(issuevaluesb);

		// bidc and bidd have values swapped, so different issuevalues.
		Map<String, Value> issuevaluesc = new HashMap<>();
		issuevaluesc.put(ISSUE1, VALUE1);
		issuevaluesc.put(ISSUE2, VALUE2);
		Bid bidc = new Bid(issuevaluesc);

		Agreements agree = new Agreements().with(bid,
				new HashSet<>(Arrays.asList(party1, party2)));
		Agreements agreeb = new Agreements().with(bidb,
				new HashSet<>(Arrays.asList(party1, party2)));
		Agreements agreec = new Agreements().with(bidc,
				new HashSet<>(Arrays.asList(party1, party2)));

		finished1 = new Finished(agree);
		finished1a = new Finished(agree);
		finished2 = new Finished(agreeb);
		finished3 = new Finished(agreec);
	}

	@Override
	public List<List<Finished>> getGeneralTestData() {
		return Arrays.asList(Arrays.asList(finished1, finished1a),
				Arrays.asList(finished2), Arrays.asList(finished3));
	}

	@Override
	public List<String> getGeneralTestStrings() {
		return Arrays.asList("Finished.*issue3=9012.*issue2=1.*issue1=\"b\".*",
				"Finished.*issue2=1.*issue1=\"b\".*",
				"Finished.*issue2=10.*issue1=\"value1\".*");
	}

	@Test
	public void nullTest() throws URISyntaxException {
		new Finished(null);
	}

	@Test
	public void testSerialize() throws JsonProcessingException {
		ObjectMapper jackson = new ObjectMapper();

		String json = jackson.writeValueAsString(finished1);
		System.out.println(json);
		assertEquals(asJson, json);
	}

	@Test
	public void testDeserialize()
			throws JsonParseException, JsonMappingException, IOException {
		// FIXME there seems a disturbing bug in this test,
		// jackson seems to ignore the @JsonProperty annotation and therefore
		// this test does not work in some cases.
		ObjectMapper jackson = new ObjectMapper();
		Finished p = (Finished) jackson.readValue(asJson, Inform.class);
		System.out.println(p);
		assertEquals(finished1, p);
	}

}
